@@ -406,7 +406,7 @@ clearlooks_gummy_draw_progressbar_fill (cairo_t *cr,
 	ge_shade_color (&colors->spot[1], 0.92, &e);
 	pattern = cairo_pattern_create_linear (0, 0, 0, height);
 	cairo_pattern_add_color_stop_rgb (pattern, 0.0, a.r, a.g, a.b);
-      	cairo_pattern_add_color_stop_rgb (pattern, 0.5, b.r, b.g, b.b);
+	cairo_pattern_add_color_stop_rgb (pattern, 0.5, b.r, b.g, b.b);
 	cairo_pattern_add_color_stop_rgb (pattern, 0.5, colors->spot[1].r, colors->spot[1].g, colors->spot[1].b);
 	cairo_pattern_add_color_stop_rgb (pattern, 1.0, e.r, e.g, e.b);
 	cairo_set_source (cr, pattern);
@@ -917,6 +917,7 @@ clearlooks_gummy_draw_scrollbar_slider (cairo_t *cr,
 	CairoColor  hilight;
 	CairoColor  shade1, shade2, shade3;
 	cairo_pattern_t *pattern;
+	int bar_x, i;
 
 	if (scrollbar->junction & CL_JUNCTION_BEGIN)
 	{
@@ -964,16 +965,17 @@ clearlooks_gummy_draw_scrollbar_slider (cairo_t *cr,
 	cairo_fill (cr);
 	cairo_pattern_destroy (pattern);
 	
-	cairo_set_source_rgba (cr, hilight.r, hilight.g, hilight.b, 0.2);
-	ge_cairo_stroke_rectangle (cr, 1.5, 1.5, width-3, height-3);
+	if (scrollbar->has_color) {
+		cairo_set_source_rgba (cr, hilight.r, hilight.g, hilight.b, 0.2);
+		ge_cairo_stroke_rectangle (cr, 1.5, 1.5, width-3, height-3);
+	}
 
-	clearlooks_set_mixed_color (cr, border, &fill, 0.3);
+	clearlooks_set_mixed_color (cr, border, &fill, scrollbar->has_color? 0.3 : 0.2);
 	ge_cairo_stroke_rectangle (cr, 0.5, 0.5, width-1, height-1);
 
 	/* Handles */
-	int bar_x = width/2 - 4;
+	bar_x = width/2 - 4;
 	cairo_translate(cr, 0.5, 0.5);
-	int i;
 	ge_cairo_set_color (cr, border);
 	for (i=0; i<3; i++)
 	{
@@ -1058,6 +1060,7 @@ static void
 clearlooks_gummy_draw_toolbar (cairo_t *cr,
                          const ClearlooksColors          *colors,
                          const WidgetParameters          *widget,
+                         const ToolbarParameters         *toolbar,
                          int x, int y, int width, int height)
 {
 	const CairoColor *fill  = &colors->bg[GTK_STATE_NORMAL];
@@ -1071,11 +1074,13 @@ clearlooks_gummy_draw_toolbar (cairo_t *cr,
 	ge_cairo_set_color  (cr, fill);
 	cairo_paint (cr);
 	
-	/* Draw highlight */
-	cairo_move_to       (cr, 0, 0.5);
-	cairo_line_to       (cr, width-1, 0.5);
-	ge_cairo_set_color  (cr, &light);
-	cairo_stroke        (cr);
+	if (!toolbar->topmost) {
+		/* Draw highlight */
+		cairo_move_to       (cr, 0, 0.5);
+		cairo_line_to       (cr, width-1, 0.5);
+		ge_cairo_set_color  (cr, &light);
+		cairo_stroke        (cr);
+	}
 
 	/* Draw shadow */
 	cairo_move_to       (cr, 0, height-0.5);
