@@ -47,6 +47,7 @@ GType clearlooks_type_rc_style = 0;
 enum
 {
   TOKEN_SCROLLBARCOLOR = G_TOKEN_LAST + 1,
+  TOKEN_COLORIZESCROLLBAR,
   TOKEN_CONTRAST,
   TOKEN_SUNKENMENU,
   TOKEN_PROGRESSBARSTYLE,
@@ -55,9 +56,12 @@ enum
   TOKEN_LISTVIEWITEMSTYLE,
   TOKEN_ANIMATION,
   TOKEN_STYLE,
+  TOKEN_RADIUS,
   
   TOKEN_CLASSIC,
   TOKEN_GLOSSY,
+  TOKEN_INVERTED,
+  TOKEN_GUMMY,
   
   TOKEN_TRUE,
   TOKEN_FALSE
@@ -70,21 +74,25 @@ static struct
   }
 clearlooks_gtk2_rc_symbols[] =
 {
-  { "scrollbar_color",   TOKEN_SCROLLBARCOLOR  },
-  { "contrast", 		 TOKEN_CONTRAST  },
-  { "sunkenmenubar",     TOKEN_SUNKENMENU },
-  { "progressbarstyle",  TOKEN_PROGRESSBARSTYLE },
-  { "menubarstyle",      TOKEN_MENUBARSTYLE },
-  { "menuitemstyle",     TOKEN_MENUITEMSTYLE },
-  { "listviewitemstyle", TOKEN_LISTVIEWITEMSTYLE },
-  { "animation",         TOKEN_ANIMATION },
-  { "style",             TOKEN_STYLE },
+  { "scrollbar_color",    TOKEN_SCROLLBARCOLOR  },
+  { "colorize_scrollbar", TOKEN_COLORIZESCROLLBAR },
+  { "contrast",           TOKEN_CONTRAST  },
+  { "sunkenmenubar",      TOKEN_SUNKENMENU },
+  { "progressbarstyle",   TOKEN_PROGRESSBARSTYLE },
+  { "menubarstyle",       TOKEN_MENUBARSTYLE },
+  { "menuitemstyle",      TOKEN_MENUITEMSTYLE },
+  { "listviewitemstyle",  TOKEN_LISTVIEWITEMSTYLE },
+  { "animation",          TOKEN_ANIMATION },
+  { "style",              TOKEN_STYLE },
+  { "radius",             TOKEN_RADIUS },
 
-  { "CLASSIC",           TOKEN_CLASSIC },
-  { "GLOSSY",            TOKEN_GLOSSY },
+  { "CLASSIC",            TOKEN_CLASSIC },
+  { "GLOSSY",             TOKEN_GLOSSY },
+  { "INVERTED",           TOKEN_INVERTED },
+  { "GUMMY",              TOKEN_GUMMY },
   
-  { "TRUE",	TOKEN_TRUE },
-  { "FALSE",	TOKEN_FALSE }
+  { "TRUE",	          TOKEN_TRUE },
+  { "FALSE",	          TOKEN_FALSE }
 };
 
 
@@ -119,13 +127,11 @@ clearlooks_rc_style_init (ClearlooksRcStyle *clearlooks_rc)
   clearlooks_rc->flags = 0;
   
   clearlooks_rc->contrast = 1.0;
-  clearlooks_rc->sunkenmenubar = 1;
-  clearlooks_rc->progressbarstyle = 0;
   clearlooks_rc->menubarstyle = 0;
-  clearlooks_rc->menuitemstyle = 1;
-  clearlooks_rc->listviewitemstyle = 1;
   clearlooks_rc->animation = FALSE;
+  clearlooks_rc->colorize_scrollbar = FALSE;
   clearlooks_rc->style = CL_STYLE_CLASSIC;
+  clearlooks_rc->radius = 3.0;
 }
 
 #ifdef HAVE_ANIMATION
@@ -201,13 +207,13 @@ clearlooks_gtk2_rc_parse_color(GtkSettings  *settings,
 }
 
 static guint
-clearlooks_gtk2_rc_parse_contrast(GtkSettings  *settings,
-		     GScanner     *scanner,
-		     double       *contrast)
+clearlooks_gtk2_rc_parse_double (GtkSettings  *settings,
+                                 GScanner     *scanner,
+                                 gdouble      *val)
 {
   guint token;
 
-  /* Skip 'contrast' */
+  /* Skip 'blah' */
   token = g_scanner_get_next_token(scanner);
 
   token = g_scanner_get_next_token(scanner);
@@ -218,7 +224,7 @@ clearlooks_gtk2_rc_parse_contrast(GtkSettings  *settings,
   if (token != G_TOKEN_FLOAT)
     return G_TOKEN_FLOAT;
 
-  *contrast = scanner->value.v_float;
+  *val = scanner->value.v_float;
   
   return G_TOKEN_NONE;
 }
@@ -253,7 +259,7 @@ clearlooks_gtk2_rc_parse_style (GtkSettings      *settings,
 {
   guint token;
 
-  g_assert (CL_NUM_STYLES == CL_STYLE_GLOSSY + 1); /* so that people don't forget ;-) */
+  g_assert (CL_NUM_STYLES == CL_STYLE_GUMMY + 1); /* so that people don't forget ;-) */
 
   /* Skip 'style' */
   token = g_scanner_get_next_token (scanner);
@@ -272,6 +278,12 @@ clearlooks_gtk2_rc_parse_style (GtkSettings      *settings,
       case TOKEN_GLOSSY:
         *style = CL_STYLE_GLOSSY;
         break;
+      case TOKEN_INVERTED:
+        *style = CL_STYLE_INVERTED;
+        break;
+      case TOKEN_GUMMY:
+        *style = CL_STYLE_GUMMY;
+        break;
       default:
         return TOKEN_CLASSIC;
     }
@@ -279,6 +291,31 @@ clearlooks_gtk2_rc_parse_style (GtkSettings      *settings,
   return G_TOKEN_NONE;
 }
 
+static guint
+clearlooks_gtk2_rc_parse_dummy (GtkSettings      *settings,
+                                GScanner         *scanner,
+                                gchar            *name)
+{
+  guint token;
+
+  /* Skip option */
+  token = g_scanner_get_next_token (scanner);
+
+  /* print a warning. Isn't there a way to get the string from the scanner? */
+  g_scanner_warn (scanner, "Clearlooks configuration option \"%s\" is not supported and will be ignored.", name);
+
+  /* equal sign */
+  token = g_scanner_get_next_token (scanner);
+  if (token != G_TOKEN_EQUAL_SIGN)
+    return G_TOKEN_EQUAL_SIGN;
+
+  /* eat whatever comes next */
+  token = g_scanner_get_next_token (scanner);
+
+  return G_TOKEN_NONE;
+}
+
+
 static guint
 clearlooks_rc_style_parse (GtkRcStyle *rc_style,
 			   GtkSettings  *settings,
@@ -326,30 +363,18 @@ clearlooks_rc_style_parse (GtkRcStyle *rc_style,
 	  token = clearlooks_gtk2_rc_parse_color (settings, scanner, &clearlooks_style->scrollbar_color);
 	  clearlooks_style->flags |= CL_FLAG_SCROLLBAR_COLOR;
 	  break;
+	case TOKEN_COLORIZESCROLLBAR:
+	  token = clearlooks_gtk2_rc_parse_boolean (settings, scanner, &clearlooks_style->colorize_scrollbar);
+	  clearlooks_style->flags |= CL_FLAG_COLORIZE_SCROLLBAR;
+	  break;
 	case TOKEN_CONTRAST:
-	  token = clearlooks_gtk2_rc_parse_contrast (settings, scanner, &clearlooks_style->contrast);
+	  token = clearlooks_gtk2_rc_parse_double (settings, scanner, &clearlooks_style->contrast);
 	  clearlooks_style->flags |= CL_FLAG_CONTRAST;
 	  break;
-	case TOKEN_SUNKENMENU:
-	  token = clearlooks_gtk2_rc_parse_int (settings, scanner, &clearlooks_style->sunkenmenubar);
-	  clearlooks_style->flags |= CL_FLAG_SUNKENMENUBAR;
-	  break;
-	case TOKEN_PROGRESSBARSTYLE:
-	  token = clearlooks_gtk2_rc_parse_int (settings, scanner, &clearlooks_style->progressbarstyle);
-	  clearlooks_style->flags |= CL_FLAG_PROGRESSBARSTYLE;
-	  break;
 	case TOKEN_MENUBARSTYLE:
 	  token = clearlooks_gtk2_rc_parse_int (settings, scanner, &clearlooks_style->menubarstyle);
 	  clearlooks_style->flags |= CL_FLAG_MENUBARSTYLE;
 	  break;
-	case TOKEN_MENUITEMSTYLE:
-	  token = clearlooks_gtk2_rc_parse_int (settings, scanner, &clearlooks_style->menuitemstyle);
-	  clearlooks_style->flags |= CL_FLAG_MENUITEMSTYLE;
-	  break;
-	case TOKEN_LISTVIEWITEMSTYLE:
-	  token = clearlooks_gtk2_rc_parse_int (settings, scanner, &clearlooks_style->listviewitemstyle);
-	  clearlooks_style->flags |= CL_FLAG_LISTVIEWITEMSTYLE;
-	  break;
 	case TOKEN_ANIMATION:
 	  token = clearlooks_gtk2_rc_parse_boolean (settings, scanner, &clearlooks_style->animation);
 	  clearlooks_style->flags |= CL_FLAG_ANIMATION;
@@ -358,6 +383,24 @@ clearlooks_rc_style_parse (GtkRcStyle *rc_style,
 	  token = clearlooks_gtk2_rc_parse_style (settings, scanner, &clearlooks_style->style);
 	  clearlooks_style->flags |= CL_FLAG_STYLE;
 	  break;
+	case TOKEN_RADIUS:
+	  token = clearlooks_gtk2_rc_parse_double (settings, scanner, &clearlooks_style->radius);
+	  clearlooks_style->flags |= CL_FLAG_RADIUS;
+	  break;
+
+	/* stuff to ignore */
+	case TOKEN_SUNKENMENU:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "sunkenmenu");
+	  break;
+	case TOKEN_PROGRESSBARSTYLE:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "progressbarstyle");
+	  break;
+	case TOKEN_MENUITEMSTYLE:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "menuitemstyle");
+	  break;
+	case TOKEN_LISTVIEWITEMSTYLE:
+	  token = clearlooks_gtk2_rc_parse_dummy (settings, scanner, "listviewitemstyle");
+	  break;
 	
 	default:
 	  g_scanner_get_next_token(scanner);
@@ -383,6 +426,7 @@ clearlooks_rc_style_merge (GtkRcStyle *dest,
 			   GtkRcStyle *src)
 {
 	ClearlooksRcStyle *dest_w, *src_w;
+	ClearlooksRcFlags flags;
 	
 	clearlooks_parent_rc_class->merge (dest, src);
 	
@@ -391,25 +435,23 @@ clearlooks_rc_style_merge (GtkRcStyle *dest,
 	
 	src_w = CLEARLOOKS_RC_STYLE (src);
 	dest_w = CLEARLOOKS_RC_STYLE (dest);
-	
-	if (src_w->flags & CL_FLAG_STYLE)
+
+	flags = (~dest_w->flags) & src_w->flags;
+
+	if (flags & CL_FLAG_STYLE)
 		dest_w->style = src_w->style;
-	if (src_w->flags & CL_FLAG_CONTRAST)
-		dest_w->contrast          = src_w->contrast;
-	if (src_w->flags & CL_FLAG_SUNKENMENUBAR)
-		dest_w->sunkenmenubar     = src_w->sunkenmenubar;
-	if (src_w->flags & CL_FLAG_PROGRESSBARSTYLE)
-		dest_w->progressbarstyle  = src_w->progressbarstyle;
-	if (src_w->flags & CL_FLAG_MENUBARSTYLE)
-		dest_w->menubarstyle      = src_w->menubarstyle;
-	if (src_w->flags & CL_FLAG_MENUITEMSTYLE)
-		dest_w->menuitemstyle     = src_w->menuitemstyle;
-	if (src_w->flags & CL_FLAG_LISTVIEWITEMSTYLE)
-		dest_w->listviewitemstyle = src_w->listviewitemstyle;
-	if (src_w->flags & CL_FLAG_SCROLLBAR_COLOR)
+	if (flags & CL_FLAG_CONTRAST)
+		dest_w->contrast = src_w->contrast;
+	if (flags & CL_FLAG_MENUBARSTYLE)
+		dest_w->menubarstyle = src_w->menubarstyle;
+	if (flags & CL_FLAG_SCROLLBAR_COLOR)
 		dest_w->scrollbar_color = src_w->scrollbar_color;
-	if (src_w->flags & CL_FLAG_ANIMATION)
+	if (flags & CL_FLAG_COLORIZE_SCROLLBAR)
+		dest_w->colorize_scrollbar = src_w->colorize_scrollbar;
+	if (flags & CL_FLAG_ANIMATION)
 		dest_w->animation = src_w->animation;
+	if (flags & CL_FLAG_RADIUS)
+		dest_w->radius = src_w->radius;
 
 	dest_w->flags |= src_w->flags;
 }
