@@ -12,7 +12,8 @@ typedef enum
 	CL_STYLE_CLASSIC = 0,
 	CL_STYLE_GLOSSY = 1,
 	CL_STYLE_INVERTED = 2,
-	CL_NUM_STYLES = 3
+	CL_STYLE_GUMMY = 3,
+	CL_NUM_STYLES = 4
 } ClearlooksStyles;
 
 
@@ -176,7 +177,7 @@ typedef struct
 	ClearlooksGapSide gap_side;
 	int gap_x;
 	int gap_width;
-	CairoColor *border; /* maybe changes this to some other hint ... */
+	const CairoColor *border; /* maybe changes this to some other hint ... */
 } FrameParameters;
 
 typedef struct
@@ -243,6 +244,11 @@ typedef struct
 	ClearlooksDirection direction;
 } ArrowParameters;
 
+typedef struct
+{
+	boolean  topmost;
+} ToolbarParameters;
+
 struct _ClearlooksStyleFunctions
 {
 	void (*draw_button)           (cairo_t				*cr,
@@ -291,24 +297,23 @@ struct _ClearlooksStyleFunctions
 	void (*draw_optionmenu)       (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
-	                               const OptionMenuParameters	*optionmenu,
+	                               const OptionMenuParameters *optionmenu,
 	                               int x, int y, int width, int height);
 					   
-	void (*draw_inset)            (cairo_t				*cr,
-                                   const ClearlooksColors		*colors,
-	                               int width, int height,
-	                               double radius, uint8 corners);
+	void (*draw_inset)            (cairo_t *cr, CairoColor bg_color, 
+                                  double x, double y, double w, double h,
+                                  double radius, uint8 corners);
 
 	void (*draw_menubar)          (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
-	                               const MenuBarParameters		*menubar,
+	                               const MenuBarParameters	*menubar,
 	                               int x, int y, int width, int height);
 
 	void (*draw_tab)              (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
-	                               const TabParameters		*tab,
+	                               const TabParameters		   *tab,
 	                               int x, int y, int width, int height);
 
 	void (*draw_frame)            (cairo_t				*cr,
@@ -318,7 +323,7 @@ struct _ClearlooksStyleFunctions
 	                               int x, int y, int width, int height);
 
 	void (*draw_separator)        (cairo_t				*cr,
-	                               const ClearlooksColors		*colors,
+	                               const CairoColor		      *color,
 	                               const WidgetParameters		*widget,
 	                               const SeparatorParameters	*separator,
 	                               int x, int y, int width, int height);
@@ -332,6 +337,7 @@ struct _ClearlooksStyleFunctions
 	void (*draw_toolbar)          (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
+	                               const ToolbarParameters          *toolbar,
 	                               int x, int y, int width, int height);
 
 	void (*draw_menuitem)         (cairo_t				*cr,
@@ -350,9 +356,9 @@ struct _ClearlooksStyleFunctions
 	                               int x, int y, int width, int height);
 
 	void (*draw_scrollbar_stepper) (cairo_t				*cr,
-	                                const ClearlooksColors		*colors,
-	                                const WidgetParameters		*widget,
-	                                const ScrollBarParameters	*scrollbar,
+	                                const ClearlooksColors	*colors,
+	                                const WidgetParameters	*widget,
+	                                const ScrollBarParameters *scrollbar,
 	                                const ScrollBarStepperParameters *stepper,
 	                                int x, int y, int width, int height);
 
@@ -378,6 +384,11 @@ struct _ClearlooksStyleFunctions
 	                               const WidgetParameters		*widget,
 	                               int x, int y, int width, int height);
 
+	void (*draw_tooltip)          (cairo_t				*cr,
+	                               const ClearlooksColors		*colors,
+	                               const WidgetParameters		*widget,
+	                               int x, int y, int width, int height);
+
 	void (*draw_handle)           (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
@@ -391,28 +402,28 @@ struct _ClearlooksStyleFunctions
 	                               int x, int y, int width, int height);
 
 	void (*draw_arrow)      (cairo_t				*cr,
-					                       const ClearlooksColors          *colors,
-					                       const WidgetParameters          *widget,
-					                       const ArrowParameters           *arrow,
+					                       const ClearlooksColors *colors,
+					                       const WidgetParameters *widget,
+					                       const ArrowParameters  *arrow,
 					                       int x, int y, int width, int height);
 	                               
 	void (*draw_checkbox)         (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
-	                               const CheckboxParameters		*checkbox,
+	                               const CheckboxParameters	*checkbox,
 	                               int x, int y, int width, int height);
 
 	void (*draw_radiobutton)      (cairo_t				*cr,
 	                               const ClearlooksColors		*colors,
 	                               const WidgetParameters		*widget,
-	                               const CheckboxParameters		*checkbox,
+	                               const CheckboxParameters	*checkbox,
 	                               int x, int y, int width, int height);
 
 	/* Style internal functions */
 	/* XXX: Only used by slider_button, inline it? */
 	void (*draw_shadow)           (cairo_t				*cr,
-                                   const ClearlooksColors		*colors,
-	                               gfloat                            radius,
+                                  const ClearlooksColors		*colors,
+	                               gfloat         radius,
 	                               int				 width,
 	                               int				 height);
 
