@@ -18,12 +18,8 @@
 *
 **************************************************************************/
 
-/* The port to cairo does not currently support the contrast options.
- * This is because I did not see the option, I'll need to investigate
- * later on what it does exactly and how to implement it.
- */
-
-#include "industrial_style_versioned_include.h"
+#include "industrial_style.h"
+#include "industrial_rc_style.h"
 #include <string.h>
 #include <stdlib.h>
 #include <ctype.h>
@@ -31,23 +27,23 @@
 #include <cairo.h>
 #include <math.h>
 
-#define WIDGET_TYPE_NAME(xx) (widget && !strcmp (G_OBJECT_TYPE_NAME (widget), (xx)))
-/* #define DEBUG 1 */
-
-#define DEFAULT_SHADOW_SIZE 3
-
 #define LINE_OPACITY 0.4
 #define HANDLE_OPACITY 0.38
 #define BUTTON_DEPRESSED_SHADOW_OPACITY_1 0.14
 #define BUTTON_DEPRESSED_SHADOW_OPACITY_2 0.07
 #define DEFAULT_SHADOW_OPACITY 0.26
-#define MENUITEM_BORDER_OPACITY 0.8
+#define MENUITEM_BORDER_OPACITY 0.38
 #define STANDARD_BORDER_OPACITY 0.38
 #define BUTTON_BORDER_OPACITY 0.38
 #define TROUGH_BG_OPACITY 0.15
 #define TROUGH_BORDER_OPACITY 0.21
 #define MENUBAR_BORDER_OPACITY 0.07
 
+#define GET_ROUNDED_BUTTONS(style) (INDUSTRIAL_STYLE (style)->rounded_buttons)
+#define GET_REAL_OPACITY(style, orig_opacity) (CLAMP ((orig_opacity) * INDUSTRIAL_STYLE (style)->contrast, 0.0, 1.0))
+
+#define IF_ROUNDED(style, rounded, otherwise) (GET_ROUNDED_BUTTONS (style) ? rounded : otherwise)
+
 static GtkStyleClass *parent_class;
 
 static void
@@ -56,76 +52,95 @@ draw_rounded_gradient (cairo_t    *cr,
 		       gint	   y,
 		       gint        width,
 		       gint        height,
+		       gfloat      gradient_width,
 		       gfloat      inner_radius,
 		       gfloat      outer_radius,
 		       CairoColor *inner_color,
 		       CairoColor *outer_color)
 {
-	gfloat gradient_width = outer_radius - inner_radius;
 	cairo_pattern_t *pattern;
 	cairo_matrix_t matrix;
+	
 	cairo_save (cr);
 	cairo_translate (cr, x, y);
 
-	/* setup a clip region for the gradient area */
-	cairo_new_path (cr);
+	/* either we need an inner_radius of 0px, or gradient_width must be
+	 * exactly outer_radius - inner_radius. I have no idea how I could
+	 * handle the other cases. (and I don't need them) */
+	if (gradient_width == -1) {
+		gradient_width = outer_radius - inner_radius;
+	} else {
+		/* we can't handle a specific gradient width and inner_radius */
+		if (gradient_width != outer_radius - inner_radius) {
+			inner_radius = 0;
+		}
+	}
 
-	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
+	if (inner_radius != 0.0 || outer_radius != 0.0) {
+		/* setup a clip region for the gradient area, this is important for the edges. */
+		cairo_save (cr);
+		
+		cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
 
-	ge_cairo_rounded_rectangle (cr, 0, 0, width, height, outer_radius, CR_CORNER_ALL);
-	ge_cairo_rounded_rectangle (cr, gradient_width, gradient_width,
-				    width - 2 * gradient_width,
-				    height - 2 * gradient_width,
-				    inner_radius, CR_CORNER_ALL);
+		ge_cairo_rounded_rectangle (cr, 0, 0, width, height, outer_radius, CR_CORNER_ALL);
+		ge_cairo_rounded_rectangle (cr, gradient_width, gradient_width,
+					    width - 2 * gradient_width,
+					    height - 2 * gradient_width,
+					    inner_radius, CR_CORNER_ALL);
 
-	cairo_clip (cr);
+		cairo_clip (cr);
 
-	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_WINDING);
+		cairo_set_fill_rule (cr, CAIRO_FILL_RULE_WINDING);
 
-	/* create radial gradient */
-	pattern = cairo_pattern_create_radial (0, 0, 0, 0, 0, outer_radius);
+		/* create radial gradient */
+		pattern = cairo_pattern_create_radial (0, 0, 0, 0, 0, outer_radius);
 
-	/* XXX: Shouldn't it work to set the first radius in create_radial
-	 * to inner_radius instead of this hack? */
-	ge_cairo_pattern_add_color_stop_color (pattern, 0, inner_color);
-	ge_cairo_pattern_add_color_stop_color (pattern,
-					       inner_radius / outer_radius,
-					       inner_color);
-	ge_cairo_pattern_add_color_stop_color (pattern, 1, outer_color);
-	cairo_set_source (cr, pattern);
-	cairo_pattern_destroy (pattern);
+		/* XXX: Shouldn't it work to set the first radius in create_radial
+		 * to inner_radius instead of this hack? */
+		ge_cairo_pattern_add_color_stop_color (pattern, 0, inner_color);
+		ge_cairo_pattern_add_color_stop_color (pattern,
+						       inner_radius / outer_radius,
+						       inner_color);
+		ge_cairo_pattern_add_color_stop_color (pattern, 1, outer_color);
+		cairo_set_source (cr, pattern);
+		cairo_pattern_destroy (pattern);
 
-	/* move to each corner, and draw it. (as everything is clipped, this is OK to do) */
-	cairo_save (cr);
-	cairo_rectangle (cr, 0, 0, outer_radius, outer_radius);
-	cairo_matrix_init_translate (&matrix, -outer_radius, -outer_radius);
-	cairo_pattern_set_matrix (pattern, &matrix);
-	cairo_fill (cr);
-	cairo_restore (cr);
+		/* move to each corner, and draw it. (as everything is clipped, this is OK to do) */
+		cairo_save (cr);
+		cairo_rectangle (cr, 0, 0, outer_radius, outer_radius);
+		cairo_matrix_init_translate (&matrix, -outer_radius, -outer_radius);
+		cairo_pattern_set_matrix (pattern, &matrix);
+		cairo_fill (cr);
+		cairo_restore (cr);
 
-	cairo_save (cr);
-	cairo_rectangle (cr, width - outer_radius, 0, outer_radius, outer_radius);
-	cairo_matrix_init_translate (&matrix, outer_radius - width, -outer_radius);
-	cairo_pattern_set_matrix (pattern, &matrix);
-	cairo_fill (cr);
-	cairo_restore (cr);
+		cairo_save (cr);
+		cairo_rectangle (cr, width - outer_radius, 0, outer_radius, outer_radius);
+		cairo_matrix_init_translate (&matrix, outer_radius - width, -outer_radius);
+		cairo_pattern_set_matrix (pattern, &matrix);
+		cairo_fill (cr);
+		cairo_restore (cr);
 
-	cairo_save (cr);
-	cairo_rectangle (cr, width - outer_radius, height - outer_radius,
-			 outer_radius, outer_radius);
-	cairo_matrix_init_translate (&matrix, outer_radius - width,
-				     outer_radius - height);
-	cairo_pattern_set_matrix (pattern, &matrix);
-	cairo_fill (cr);
-	cairo_restore (cr);
+		cairo_save (cr);
+		cairo_rectangle (cr, width - outer_radius, height - outer_radius,
+				 outer_radius, outer_radius);
+		cairo_matrix_init_translate (&matrix, outer_radius - width,
+					     outer_radius - height);
+		cairo_pattern_set_matrix (pattern, &matrix);
+		cairo_fill (cr);
+		cairo_restore (cr);
 
-	cairo_save (cr);
-	cairo_rectangle (cr, 0, height - outer_radius, outer_radius, outer_radius);
-	cairo_matrix_init_translate (&matrix, -outer_radius, outer_radius - height);
-	cairo_pattern_set_matrix (pattern, &matrix);
-	cairo_fill (cr);
-	cairo_restore (cr);
+		cairo_save (cr);
+		cairo_rectangle (cr, 0, height - outer_radius, outer_radius, outer_radius);
+		cairo_matrix_init_translate (&matrix, -outer_radius, outer_radius - height);
+		cairo_pattern_set_matrix (pattern, &matrix);
+		cairo_fill (cr);
+		cairo_restore (cr);
+		
+		cairo_restore (cr);
+	}
 
+	/* The _rounded_ part of the corners are drawn at this point ... */
+	
 	/* create linear gradient */
 	pattern = cairo_pattern_create_linear (0, 0, 0, gradient_width);
 	ge_cairo_pattern_add_color_stop_color (pattern, 0, inner_color);
@@ -135,26 +150,58 @@ draw_rounded_gradient (cairo_t    *cr,
 
 	/* draw those gradients */
 	cairo_save (cr);
-	cairo_rectangle (cr, outer_radius, 0, width - 2 * outer_radius,
-			 gradient_width);
-	cairo_matrix_init_rotate (&matrix, M_PI);
+	
+	/* This is not perfect, as drawing twice with 0.5 opacity is not the same
+	 * as drawing once with full opacity ... */
+	/* TOP */
+	cairo_move_to (cr, outer_radius, 0);
+	cairo_line_to (cr, outer_radius, outer_radius);
+	cairo_line_to (cr, gradient_width, gradient_width);
+	cairo_line_to (cr, width - gradient_width, gradient_width);
+	cairo_line_to (cr, width - outer_radius, outer_radius);
+	cairo_line_to (cr, width - outer_radius, 0);
+	cairo_close_path (cr);
+	
+	/* cairo_rectangle (cr, outer_radius, 0, width - 2 * outer_radius,
+			 gradient_width); */
+	cairo_matrix_init_rotate (&matrix, G_PI);
 	cairo_matrix_translate (&matrix, 0, -gradient_width);
 	cairo_pattern_set_matrix (pattern, &matrix);
 	cairo_fill (cr);
 	cairo_restore (cr);
 
 	cairo_save (cr);
-	cairo_rectangle (cr, width - outer_radius, outer_radius, outer_radius,
-			 height - 2 * outer_radius);
-	cairo_matrix_init_rotate (&matrix, M_PI_2);
+	
+	/* RIGHT */
+	cairo_move_to (cr, width, outer_radius);
+	cairo_line_to (cr, width - outer_radius, outer_radius);
+	cairo_line_to (cr, width - gradient_width, gradient_width);
+	cairo_line_to (cr, width - gradient_width, height - gradient_width);
+	cairo_line_to (cr, width - outer_radius, height - outer_radius);
+	cairo_line_to (cr, width, height - outer_radius);
+	cairo_close_path (cr);
+	
+	/* cairo_rectangle (cr, width - outer_radius, outer_radius, outer_radius,
+			 height - 2 * outer_radius); */
+	cairo_matrix_init_rotate (&matrix, G_PI_2);
 	cairo_matrix_translate (&matrix, -width + gradient_width, -gradient_width);
 	cairo_pattern_set_matrix (pattern, &matrix);
 	cairo_fill (cr);
 	cairo_restore (cr);
 
 	cairo_save (cr);
-	cairo_rectangle (cr, outer_radius, height - gradient_width,
-			 width - 2 * outer_radius, gradient_width);
+
+	/* BOTTOM */
+	cairo_move_to (cr, outer_radius, height);
+	cairo_line_to (cr, outer_radius, height - outer_radius);
+	cairo_line_to (cr, gradient_width, height - gradient_width);
+	cairo_line_to (cr, width - gradient_width, height - gradient_width);
+	cairo_line_to (cr, width - outer_radius, height - outer_radius);
+	cairo_line_to (cr, width - outer_radius, height);
+	cairo_close_path (cr);
+
+	/* cairo_rectangle (cr, outer_radius, height - gradient_width,
+			 width - 2 * outer_radius, gradient_width); */
 	cairo_matrix_init_rotate (&matrix, 0);
 	cairo_matrix_translate (&matrix, 0, -height + gradient_width);
 	cairo_pattern_set_matrix (pattern, &matrix);
@@ -162,9 +209,18 @@ draw_rounded_gradient (cairo_t    *cr,
 	cairo_restore (cr);
 
 	cairo_save (cr);
-	cairo_rectangle (cr, 0, outer_radius, gradient_width,
-			 height - 2 * outer_radius);
-	cairo_matrix_init_rotate (&matrix, -M_PI_2);
+	/* LEFT */
+	cairo_move_to (cr, 0, height - outer_radius);
+	cairo_line_to (cr, outer_radius, height - outer_radius);
+	cairo_line_to (cr, gradient_width, height - gradient_width);
+	cairo_line_to (cr, gradient_width, gradient_width);
+	cairo_line_to (cr, outer_radius, outer_radius);
+	cairo_line_to (cr, 0, outer_radius);
+	cairo_close_path (cr);
+
+	/* cairo_rectangle (cr, 0, outer_radius, gradient_width,
+			 height - 2 * outer_radius); */
+	cairo_matrix_init_rotate (&matrix, -G_PI_2);
 	cairo_matrix_translate (&matrix, -gradient_width, 0);
 	cairo_pattern_set_matrix (pattern, &matrix);
 	cairo_fill (cr);
@@ -176,26 +232,43 @@ draw_rounded_gradient (cairo_t    *cr,
 /* The fill will only be drawn if fill != NULL.
  * radius should be 1.5/2.5 for 1px/2px corners. */
 static void
-draw_rounded_rect_cairo (cairo_t    *cr,
-			 gint        x,
-			 gint        y,
-			 gint        width,
-			 gint        height,
-			 gdouble     radius,
-			 CairoColor *bevel,
-			 CairoColor *fill)
+draw_rounded_rect (cairo_t     *cr,
+		   gint         x,
+		   gint         y,
+		   gint         width,
+		   gint         height,
+		   gdouble      radius,
+		   CairoColor  *bevel,
+		   CairoColor  *fill,
+		   CairoCorners corners)
 {
+	CairoColor real_bevel = *bevel;
+	
 	if (fill) {
-		ge_cairo_rounded_rectangle (cr, x, y, width, height, radius,
-					    CR_CORNER_ALL);
+		/* This fill is a little optimized in most cases, to only draw a pixel aligned rect.
+		 * To prevent errors, we need to adjust the bevel color ... */
+		
+		if (radius <= 2.5 && fill->a == 1.0) {
+			real_bevel.a = 1;
+			real_bevel.r = fill->r * (1.0 - bevel->a) + bevel->r * bevel->a;
+			real_bevel.g = fill->g * (1.0 - bevel->a) + bevel->g * bevel->a;
+			real_bevel.b = fill->b * (1.0 - bevel->a) + bevel->b * bevel->a;
+
+			/* At least with a radius like this, the corner pixel will still be
+			 * painted by the stroke ... using a rect here should be fine. */
+			cairo_rectangle (cr, x + 1, y + 1, width -2, height - 2);
+		} else {
+			ge_cairo_rounded_rectangle (cr, x, y, width, height, radius,
+						    corners);
+		}
 
 		ge_cairo_set_color (cr, fill);
 		cairo_fill (cr);
 	}
 
-	ge_cairo_set_color (cr, bevel);
+	ge_cairo_set_color (cr, &real_bevel);
 	ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5, width - 1, height - 1,
-				    radius, CR_CORNER_ALL);
+				    radius, corners);
 	cairo_stroke (cr);
 }
 
@@ -213,11 +286,15 @@ draw_grid_cairo (cairo_t    *cr,
 {
 	gint xpos, ypos;
 	gboolean indented = FALSE;
+	CairoColor c = *color;
+	gfloat alpha1 = color->a;
+	gfloat alpha2 = color->a * 0.5;
+	
 	cairo_save (cr);
 
-	ge_cairo_set_color (cr, color);
-
 	/* walk trough the rows */
+	/* for speed reasons, replaced with code to paint the 5 pixel manually.
+	 * First paint the center ones, then the others */
 	for (ypos = y + 1; ypos < y + height; ypos += 2) {
 		xpos = x;
 		if (!indented)
@@ -226,15 +303,35 @@ draw_grid_cairo (cairo_t    *cr,
 			xpos += 3;
 
 		for (; xpos < x + width; xpos += 4) {
-			/* This one would fill in the center pixel exactly, but
-			 * the dots don't look as good then.
-			 * cairo_arc (cr, xpos + 0.5, ypos + 0.5, sqrt (0.5), 0, 2 * M_PI); */
-			cairo_arc (cr, xpos + 0.5, ypos + 0.5, sqrt (0.7), 0, 2 * M_PI);
-			cairo_fill (cr);
+			cairo_rectangle (cr, xpos, ypos, 1, 1);	
 		}
 
 		indented = !indented;
 	}
+	c.a = alpha1;
+	ge_cairo_set_color (cr, &c);
+	cairo_fill (cr);
+	
+	indented = FALSE;
+	for (ypos = y + 1; ypos < y + height; ypos += 2) {
+		xpos = x;
+		if (!indented)
+			xpos += 1;
+		else
+			xpos += 3;
+
+		for (; xpos < x + width; xpos += 4) {
+			cairo_rectangle (cr, xpos - 1, ypos, 1, 1);
+			cairo_rectangle (cr, xpos, ypos - 1, 1, 1);
+			cairo_rectangle (cr, xpos + 1, ypos, 1, 1);
+			cairo_rectangle (cr, xpos, ypos + 1, 1, 1);
+		}
+
+		indented = !indented;
+	}
+	c.a = alpha2;
+	ge_cairo_set_color (cr, &c);
+	cairo_fill (cr);
 
 	cairo_restore (cr);
 }
@@ -267,7 +364,7 @@ draw_hline (GtkStyle     *style,
 		y++;
 
 	ge_gdk_color_to_cairo (&style->fg[state_type], &color);
-	color.a = LINE_OPACITY;
+	color.a = GET_REAL_OPACITY(style, LINE_OPACITY);
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	ge_cairo_set_color (cr, &color);
@@ -317,7 +414,7 @@ draw_handle (GtkStyle      *style,
 		width -= 4;
 		height -= 4;
 	}
-
+	
 	if (shadow_type == GTK_SHADOW_NONE) {
 		maxwidth = width;
 		maxheight = height;
@@ -342,7 +439,7 @@ draw_handle (GtkStyle      *style,
 
 	ge_gdk_color_to_cairo (&style->fg[state_type], &color);
 
-	color.a = HANDLE_OPACITY;
+	color.a = GET_REAL_OPACITY (style, HANDLE_OPACITY);
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
@@ -375,7 +472,7 @@ draw_vline (GtkStyle     *style,
 	CHECK_ARGS
 	
 	ge_gdk_color_to_cairo (&style->fg[state_type], &color);
-	color.a = LINE_OPACITY;
+	color.a = GET_REAL_OPACITY (style, LINE_OPACITY);
 
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	ge_cairo_set_color (cr, &color);
@@ -416,7 +513,7 @@ draw_slider (GtkStyle      *style,
 	SANITIZE_SIZE
 	
 	/* This makes the slider overlay the stepper buttons at the sides. XXX */
-	if (widget && GTK_IS_SCROLLBAR (widget)) {
+	if (widget && GE_IS_SCROLLBAR (widget)) {
 		GtkAdjustment *adjustment = GTK_RANGE (widget)->adjustment;
 		/* If all of these values are 0, don't do anything.  This fixes mozilla/galeon scrollbars. */
 		if (adjustment->value ||
@@ -444,7 +541,6 @@ draw_slider (GtkStyle      *style,
 		}
 	}
 
-	/* XXX? */
 	gtk_paint_box (style, window, state_type, shadow_type,
 		       area, widget, detail, x, y, width, height);
 
@@ -460,7 +556,7 @@ draw_slider (GtkStyle      *style,
 	y += (height - handle_height) / 2;
 
 	ge_gdk_color_to_cairo (&style->fg[state_type], &color);
-	color.a = HANDLE_OPACITY;
+	color.a = GET_REAL_OPACITY (style, HANDLE_OPACITY);
 	cr = ge_gdk_drawable_to_cairo (window, area);
 
 	draw_grid_cairo (cr, &color, x, y, handle_width, handle_height);
@@ -483,12 +579,9 @@ real_draw_box (GtkStyle      *style,
 	       gint           height,
 	       gboolean       fill)
 {
-	/* None of the drawing routines below are set up to handle SHADOW_TYPE_NONE,
-	 * so just fill in a blank box here.  Ideally everything in the theme
-	 * would be reworked to handle SHADOW_TYPE_NONE correctly */
-	if (shadow_type == GTK_SHADOW_NONE)
-		return;
-
+	/* For the shadow_type == GTK_SHADOW_NONE case we try to do some magic ...
+	 * Buttons are drawn normally, but without any shadow effects, and most
+	 * important, for unkown stuff we fall back to flat_box. */
 	cairo_save (cr);
 
 	if (CHECK_DETAIL (detail, "button") ||
@@ -497,25 +590,30 @@ real_draw_box (GtkStyle      *style,
 		/* Draw the button border, except if this is a "default" button. */
 		CairoColor bg;
 		CairoColor fg;
+		CairoCorners corners = CR_CORNER_ALL;
 
 		ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 		ge_gdk_color_to_cairo (&style->fg[state_type], &fg);
-		fg.a = BUTTON_BORDER_OPACITY;
-
-		if (GET_ROUNDED_BUTTONS (style))
-			ge_cairo_rounded_rectangle (cr, x + 0.5, y + 0.5,
-						    width - 1, height - 1, 2.5,
-						    CR_CORNER_ALL);
+		
+		/* So we don't need to use transparency in the border,
+		 * hack this here. If it doesn't work correctly, it won't
+		 * look too bad. */
+		if (widget && GE_WIDGET_HAS_DEFAULT (widget))
+			fg.a = GET_REAL_OPACITY (style, 1.0);
 		else
-			cairo_rectangle (cr, x + 0.5, y + 0.5,
-			                 width - 1, height - 1);
+			fg.a = GET_REAL_OPACITY (style, BUTTON_BORDER_OPACITY);
 
-		ge_cairo_set_color (cr, &bg);
-		cairo_fill_preserve (cr);
 
-		cairo_set_line_width (cr, 1);
-		ge_cairo_set_color (cr, &fg);
-		cairo_stroke (cr);
+		if (ge_is_in_combo_box (widget)) {
+			if (ge_widget_is_ltr (widget))
+				corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
+			else
+				corners = CR_CORNER_TOPLEFT | CR_CORNER_BOTTOMLEFT;
+		}
+		if (!GET_ROUNDED_BUTTONS (style))
+			corners = CR_CORNER_NONE;
+
+		draw_rounded_rect (cr, x, y, width, height, 2.5, &fg, &bg, corners);
 
 		if (shadow_type == GTK_SHADOW_IN) {
 			gint slope = MIN (width / 2, height / 2);
@@ -525,8 +623,8 @@ real_draw_box (GtkStyle      *style,
 			ge_gdk_color_to_cairo (&style->fg[state_type], &inner);
 			ge_gdk_color_to_cairo (&style->fg[state_type], &outer);
 			/* XXX: some way to calculate these would be nice. */
-			inner.a = BUTTON_DEPRESSED_SHADOW_OPACITY_2;
-			outer.a = BUTTON_DEPRESSED_SHADOW_OPACITY_1;
+			inner.a = 0.0;
+			outer.a = GET_REAL_OPACITY (style, BUTTON_BORDER_OPACITY * 0.7); /* XXX */
 
 			cairo_save (cr);
 			cairo_move_to (cr, x, y + height);
@@ -537,22 +635,16 @@ real_draw_box (GtkStyle      *style,
 			cairo_close_path (cr);
 			cairo_clip (cr);
 
-			if (GET_ROUNDED_BUTTONS (style))
-				draw_rounded_gradient (cr, x + 1, y + 1,
-						       width - 2, height - 2,
-						       0.5, 2.5, &inner,
-						       &outer);
-			else
-				draw_rounded_gradient (cr, x + 1, y + 1,
-						       width - 2, height - 2,
-						       0.0, 2.0, &inner,
-						       &outer);
+			draw_rounded_gradient (cr, x + 1, y + 1,
+					       width - 2, height - 2, 3.0,
+					       0.5, IF_ROUNDED (style, 2.5, 0), &inner,
+					       &outer);
 
 			cairo_restore (cr);
 		}
 
 		/* Only draw shadow if button is large enough to fit it. */
-		if (width > 18 && height > 18) {
+		if (width > 18 && height > 18 && shadow_type != GTK_SHADOW_NONE) {
 			gint slope = MIN (width / 2, height / 2);
 			CairoColor inner;
 			CairoColor outer;
@@ -566,7 +658,7 @@ real_draw_box (GtkStyle      *style,
 			ge_gdk_color_to_cairo (&style->fg[state_type], &inner);
 			ge_gdk_color_to_cairo (&style->fg[state_type], &outer);
 			inner.a = 0;
-			outer.a = 0.1;	/* XXX: #define this? */
+			outer.a = GET_REAL_OPACITY (style, 0.1);	/* XXX: #define this? */
 
 			cairo_save (cr);
 			cairo_move_to (cr, x, y + height);
@@ -579,14 +671,9 @@ real_draw_box (GtkStyle      *style,
 
 			/* XXX: This results in a few pixel that stay blank.
 			 * Need to rethink this one. */
-			if (GET_ROUNDED_BUTTONS (style))
-				draw_rounded_gradient (cr, x + 1, y + 1,
-						       width - 2, height - 2, 1,
-						       5, &inner, &outer);
-			else
-				draw_rounded_gradient (cr, x + 1, y + 1,
-						       width - 2, height - 2, 0,
-						       4, &inner, &outer);
+			draw_rounded_gradient (cr, x + 1, y + 1,
+					       width - 2, height - 2, 4, 0,
+					       IF_ROUNDED (style, 2.5, 0), &inner, &outer);
 			cairo_restore (cr);
 		}
 
@@ -623,30 +710,28 @@ real_draw_box (GtkStyle      *style,
 		CairoColor shadow_outer;
 		CairoColor fg;
 		gfloat inner_radius;
+		gint shadow_size;
+		GtkBorder default_border;
+
+		ge_button_get_default_border (widget, &default_border);
+		shadow_size = MIN (MIN (default_border.left, default_border.right),
+		                   MIN (default_border.top, default_border.bottom));
 
 		inner_radius = GET_ROUNDED_BUTTONS (style) ? 2.5 : 0;
 
 		ge_gdk_color_to_cairo (&style->fg[state_type], &fg);
 
-		ge_cairo_rounded_rectangle (cr, x + DEFAULT_SHADOW_SIZE + 0.5,
-					    y + DEFAULT_SHADOW_SIZE + 0.5,
-					    width - 2 * DEFAULT_SHADOW_SIZE - 1,
-					    height - 2 * DEFAULT_SHADOW_SIZE -
-					    1, inner_radius, CR_CORNER_ALL);
-
-		ge_cairo_set_color (cr, &fg);
-		cairo_stroke (cr);
-
 		/* draw the shadow around this. */
 		shadow_outer = fg;
 		shadow_inner = fg;
 		shadow_outer.a = 0;
-		shadow_inner.a = DEFAULT_SHADOW_OPACITY;
+		shadow_inner.a = GET_REAL_OPACITY (style, DEFAULT_SHADOW_OPACITY);
 
-		draw_rounded_gradient (cr, x, y, width, height, inner_radius,
-				       DEFAULT_SHADOW_SIZE + inner_radius,
+		draw_rounded_gradient (cr, x, y, width, height, -1, inner_radius,
+				       shadow_size + inner_radius,
 				       &shadow_inner, &shadow_outer);
 	} else if (CHECK_DETAIL (detail, "menuitem")) {
+		CairoCorners corners;
 		CairoColor bevel;
 		CairoColor bg;
 
@@ -655,51 +740,73 @@ real_draw_box (GtkStyle      *style,
 		/* for some reason, industrial always uses GTK_STATE_NORMAL here, can't change that
 		 * without breaking compatibility. */
 		ge_gdk_color_to_cairo (&style->fg[GTK_STATE_NORMAL], &bevel);
-		bevel.a = MENUITEM_BORDER_OPACITY;
+		bevel.a = GET_REAL_OPACITY (style, MENUITEM_BORDER_OPACITY);
+
+		corners = GET_ROUNDED_BUTTONS (style) ? CR_CORNER_ALL : CR_CORNER_NONE;
 
 		if (fill)
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, &bg);
+			draw_rounded_rect (cr, x, y, width, height, 1.5,
+					   &bevel, &bg, corners);
 		else
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, NULL);
+			draw_rounded_rect (cr, x, y, width, height, 1.5,
+					   &bevel, NULL, corners);
 	} else if (CHECK_DETAIL (detail, "trough")
+		   || CHECK_DETAIL (detail, "trough-lower")
+		   || CHECK_DETAIL (detail, "trough-upper")
 		   || CHECK_DETAIL (detail, "menu")
 		   || CHECK_DETAIL (detail, "dockitem_bin")
 		   || CHECK_DETAIL (detail, "handlebox_bin")
 		   || CHECK_DETAIL (detail, "toolbar")
 		   || CHECK_DETAIL (detail, "dockitem")
-		   || WIDGET_TYPE_NAME ("PanelAppletFrame")) {
+		   || ge_object_is_a (G_OBJECT (widget), "PanelAppletFrame")) {
 		CairoColor bevel;
 		CairoColor bg;
 
 		if (fill) {
-			/* XXX: should also handle the trough invert style property ...
-			 * but what do I want to do in that case? */
-			if (CHECK_DETAIL (detail, "trough")) {
+			if (CHECK_DETAIL (detail, "trough") ||
+			    CHECK_DETAIL (detail, "trough-lower") ||
+			    CHECK_DETAIL (detail, "trough-upper")) {
+				CairoColor bg_color;
+
 				/* Troughs cannot be transparent!
 				 * This is a hack to fix this by filling with a solid color first. */
-				ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &bg);
-				ge_cairo_set_color (cr, &bg);
+				ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &bg_color);
+				ge_cairo_set_color (cr, &bg_color);
 				cairo_rectangle (cr, x, y, width, height);
 				cairo_fill (cr);
 
 				/* Troughs always draw with state ACTIVE or INSENSITIVE,
 				 * so this sets the state to NORMAL to get the correct colors. */
-				if (state_type == GTK_STATE_ACTIVE)
+				if (state_type == GTK_STATE_ACTIVE) {
 					state_type = GTK_STATE_NORMAL;
+				}
+				
+				/* XXX: This might need improvement, at least now it is "implemented" */
+				if (CHECK_DETAIL (detail, "trough-lower")) {
+					ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &bg);
+				} else {
+					ge_gdk_color_to_cairo (&style->fg[state_type], &bg);
+					
+					bg.a = GET_REAL_OPACITY (style, TROUGH_BG_OPACITY);
+					
+					/* Compose the color here instead leaving the work to
+					 * cairo/xserver which doesn't know that this is a solid
+					 * color. */
+					bg.r = bg.r * bg.a + bg_color.r * (1 - bg.a);
+					bg.g = bg.g * bg.a + bg_color.g * (1 - bg.a);
+					bg.b = bg.b * bg.a + bg_color.b * (1 - bg.a);
+					bg.a = 1.0;
+				}
 
-				ge_gdk_color_to_cairo (&style->fg[state_type], &bg);
-				bg.a = TROUGH_BG_OPACITY;
 				ge_gdk_color_to_cairo (&style->fg[state_type],  &bevel);
-				bevel.a = TROUGH_BORDER_OPACITY;
+				bevel.a = GET_REAL_OPACITY (style, TROUGH_BORDER_OPACITY);
 			} else {
 				ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 				ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
-				bevel.a = TROUGH_BORDER_OPACITY;
+				bevel.a = GET_REAL_OPACITY (style, TROUGH_BORDER_OPACITY);
 			}
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, &bg);
+			draw_rounded_rect (cr, x, y, width, height, 1.5,
+					   &bevel, &bg, IF_ROUNDED (style, CR_CORNER_ALL, CR_CORNER_NONE));
 		}
 	} else if (CHECK_DETAIL (detail, "menubar")) {
 		CairoColor bg;
@@ -707,16 +814,22 @@ real_draw_box (GtkStyle      *style,
 
 		ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 		ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
-		bevel.a = MENUBAR_BORDER_OPACITY;
+		bevel.a = GET_REAL_OPACITY (style, MENUBAR_BORDER_OPACITY);
+		
+		/* ignore the menubar if the shadow is NONE. This is the case for the
+		 * mainmenu applet and the task selector. */
+		if (shadow_type == GTK_SHADOW_NONE)
+			return;
 
 		if (fill)
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, &bg);
+			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+					   &bevel, &bg, CR_CORNER_ALL);
 		else
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, NULL);
+			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+					   &bevel, NULL, CR_CORNER_ALL);
 	} else if (CHECK_DETAIL (detail, "spinbutton_up")
 		   || CHECK_DETAIL (detail, "spinbutton_down")) {
+		/* XXX: This results in basically drawing nothing, right? */
 		if (CHECK_DETAIL (detail, "spinbutton_up")) {
 			x += 1;
 			y += 1;
@@ -741,36 +854,50 @@ real_draw_box (GtkStyle      *style,
 
 		ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 		ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
-		bevel.a = 0.5; /* XXX: #define this? */
+		bevel.a = GET_REAL_OPACITY (style, 0.5); /* XXX: #define this? */
 
 		/* XXX: fill can never be FALSE, right? */
 		if (fill)
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, &bg);
+			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+					   &bevel, &bg, CR_CORNER_ALL);
 		else
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, NULL);
+			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+					   &bevel, NULL, CR_CORNER_ALL);
 	} else {
 		CairoColor bevel, bg;
+		CairoCorners corners = CR_CORNER_ALL;
 
-		/* XXX: the "spinbutton" should not have rounded corners on the inside ... */
-		if (IS_SPIN_BUTTON (widget) && CHECK_DETAIL (detail, "entry")) {
+		if ((GE_IS_SPIN_BUTTON (widget) && CHECK_DETAIL (detail, "entry")) ||
+		    ge_is_in_combo_box (widget)) {
 			/* effectively cut one side off. */
 			width += 2;
 			if (!ge_widget_is_ltr (widget))
 				x -= 2;
 		}
+		
+		if (CHECK_DETAIL (detail, "spinbutton")) {
+			if (ge_widget_is_ltr (widget))
+				corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
+			else
+				corners = CR_CORNER_TOPLEFT | CR_CORNER_BOTTOMLEFT;
+		}
 
 		ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
+		
 		ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
-		bevel.a = STANDARD_BORDER_OPACITY;
+		bevel.a = GET_REAL_OPACITY (style, STANDARD_BORDER_OPACITY);
 
-		if (fill)
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, &bg);
-		else
-			draw_rounded_rect_cairo (cr, x, y, width, height, 1.5,
-						 &bevel, NULL);
+		if (fill) {
+			if (shadow_type != GTK_SHADOW_NONE) {
+				draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+						   &bevel, &bg, corners);
+			} else {
+				parent_class->draw_flat_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
+			}
+		} else if (shadow_type != GTK_SHADOW_NONE) {
+			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+					   &bevel, NULL, corners);
+		}
 	}
 
 	cairo_restore (cr);
@@ -794,7 +921,10 @@ draw_box (GtkStyle      *style,
 	printf ("draw_box: %p %p %s %i %i %i %i\n", widget, window, detail, x,
 		y, width, height);
 #endif
-	CHECK_ARGS SANITIZE_SIZE cr = ge_gdk_drawable_to_cairo (window, area);
+	CHECK_ARGS
+	SANITIZE_SIZE
+	
+	cr = ge_gdk_drawable_to_cairo (window, area);
 
 	real_draw_box (style, cr, window, state_type, shadow_type, area, widget,
 		       detail, x, y, width, height, TRUE);
@@ -802,6 +932,50 @@ draw_box (GtkStyle      *style,
 	cairo_destroy (cr);
 }
 
+static void
+draw_focus (GtkStyle      *style,
+            GdkWindow     *window,
+            GtkStateType   state_type,
+            GdkRectangle  *area,
+            GtkWidget     *widget,
+            const gchar   *detail,
+            gint           x,
+            gint           y,
+            gint           width,
+            gint           height)
+{
+	/* we draw a custom focus for entries ... */
+	if (CHECK_DETAIL (detail, "entry")) {
+		cairo_t *cr;
+		CairoColor border;
+		CairoCorners corners = CR_CORNER_ALL;
+		
+		SANITIZE_SIZE
+		CHECK_ARGS
+
+		if (ge_is_in_combo_box (widget) || GE_IS_SPIN_BUTTON (widget)) {
+			if (ge_widget_is_ltr (widget))
+				corners = CR_CORNER_TOPLEFT | CR_CORNER_BOTTOMLEFT;
+			else
+				corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
+		}
+		
+		cr = ge_gdk_drawable_to_cairo (window, area);
+		ge_gdk_color_to_cairo (&style->bg[GTK_STATE_SELECTED], &border);
+		
+		ge_cairo_rounded_rectangle (cr, x + 1, y + 1, width - 2, height - 2, IF_ROUNDED (style, 1.5, 0), corners);
+		ge_cairo_set_color (cr, &border);
+		cairo_set_line_width (cr, 2.0);
+		cairo_stroke (cr);	
+		
+		cairo_destroy (cr);
+		
+		return;
+	}
+	
+	parent_class->draw_focus (style, window, state_type, area, widget, detail, x, y, width, height);
+}
+
 static void
 draw_shadow (GtkStyle      *style,
 	     GdkWindow     *window,
@@ -820,7 +994,10 @@ draw_shadow (GtkStyle      *style,
 	printf ("draw_shadow: %p %p %s %i %i %i %i\n", widget, window, detail,
 		x, y, width, height);
 #endif
-	CHECK_ARGS SANITIZE_SIZE cr = ge_gdk_drawable_to_cairo (window, area);
+	CHECK_ARGS
+	SANITIZE_SIZE
+	
+	cr = ge_gdk_drawable_to_cairo (window, area);
 
 	real_draw_box (style, cr, window, state_type, shadow_type, area, widget,
 		       detail, x, y, width, height, FALSE);
@@ -879,7 +1056,7 @@ real_draw_box_gap (GtkStyle       *style,
 
 	ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 	ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
-	bevel.a = STANDARD_BORDER_OPACITY;
+	bevel.a = GET_REAL_OPACITY (style, STANDARD_BORDER_OPACITY);
 
 	/* clip the gap area */
 	cairo_save (cr);
@@ -910,9 +1087,11 @@ real_draw_box_gap (GtkStyle       *style,
 	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_WINDING);
 
 	if (fill)
-		draw_rounded_rect_cairo (cr, x, y, width, height, 1.5, &bevel, &bg);
+		draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+		                   &bevel, &bg, CR_CORNER_ALL);
 	else
-		draw_rounded_rect_cairo (cr, x, y, width, height, 1.5, &bevel, NULL);
+		draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+		                   &bevel, NULL, CR_CORNER_ALL);
 	cairo_restore (cr);
 
 	/* and draw two dots on both sides of the gap. */
@@ -1027,6 +1206,7 @@ draw_extension (GtkStyle       *style,
 		gint            height,
 		GtkPositionType gap_side)
 {
+	CairoCorners corners = CR_CORNER_ALL;
 	cairo_t *cr;
 	cairo_pattern_t *pattern = NULL;
 	CairoColor bevel;
@@ -1040,40 +1220,47 @@ draw_extension (GtkStyle       *style,
 	if (shadow_type == GTK_SHADOW_NONE)
 		return;
 
+	corners = GET_ROUNDED_BUTTONS (style) ? CR_CORNER_ALL : CR_CORNER_NONE;
+
 	cr = ge_gdk_drawable_to_cairo (window, area);
 	cairo_rectangle (cr, x, y, width, height);
 	cairo_clip (cr);
 
 	ge_gdk_color_to_cairo (&style->bg[state_type], &bg);
 	ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
-	bevel.a = STANDARD_BORDER_OPACITY;
+	bevel.a = GET_REAL_OPACITY (style, STANDARD_BORDER_OPACITY);
 
 	switch (gap_side) {
 	case GTK_POS_TOP:
-		draw_rounded_rect_cairo (cr, x, y - 3, width, height + 3, 1.5, &bevel, &bg);
+		draw_rounded_rect (cr, x, y - 3, width, height + 3, 1.5,
+		                   &bevel, &bg, corners);
 		pattern = cairo_pattern_create_linear (x, y, x, y + 4);
 		cairo_rectangle (cr, x, y, width, 4);
 		break;
 	case GTK_POS_BOTTOM:
-		draw_rounded_rect_cairo (cr, x, y, width, height + 3, 1.5, &bevel, &bg);
+		draw_rounded_rect (cr, x, y, width, height + 3, 1.5,
+		                   &bevel, &bg, corners);
 		pattern = cairo_pattern_create_linear (x, y + height, x, y + height - 4);
 		cairo_rectangle (cr, x, y + height - 4, width, 4);
 		break;
 	case GTK_POS_LEFT:
-		draw_rounded_rect_cairo (cr, x - 3, y, width + 3, height, 1.5, &bevel, &bg);
+		draw_rounded_rect (cr, x - 3, y, width + 3, height, 1.5,
+		                   &bevel, &bg, corners);
 		pattern = cairo_pattern_create_linear (x, y, x + 4, y);
 		cairo_rectangle (cr, x, y, 4, height);
 		break;
 	case GTK_POS_RIGHT:
-		draw_rounded_rect_cairo (cr, x, y, width + 3, height, 1.5, &bevel, &bg);
+		draw_rounded_rect (cr, x, y, width + 3, height, 1.5,
+		                   &bevel, &bg, corners);
 		pattern = cairo_pattern_create_linear (x + width, y, x + width - 4, y);
 		cairo_rectangle (cr, x + width - 4, y, 4, height);
 		break;
 	}
 
 	if (state_type != GTK_STATE_NORMAL) {
-		bevel.a = 0.15;
+		bevel.a *= 0.3;
 		ge_cairo_pattern_add_color_stop_color (pattern, 0, &bevel);
+
 		bevel.a = 0;
 		ge_cairo_pattern_add_color_stop_color (pattern, 1, &bevel);
 
@@ -1120,27 +1307,39 @@ draw_check (GtkStyle * style,
 	width = check_size;
 	height = check_size;
 
-	fg.a = DEFAULT_SHADOW_OPACITY;
-	draw_rounded_rect_cairo (cr, x, y, width, height, 1.5, &fg, &bg);
+	fg.a = GET_REAL_OPACITY (style, DEFAULT_SHADOW_OPACITY);
+	draw_rounded_rect (cr, x, y, width, height, 1.5,
+	                   &fg, &bg, GET_ROUNDED_BUTTONS (style) ? CR_CORNER_ALL : CR_CORNER_NONE);
+
+	cairo_save (cr);
+	fg.a = fg.a * 0.3;
+	
+	cairo_move_to (cr, x, y + check_size);
+	cairo_line_to (cr, x + check_size, y);
+	cairo_line_to (cr, x, y);
+	cairo_close_path (cr);
+	cairo_clip (cr);
+	
+	draw_rounded_rect (cr, x + 1, y + 1, width - 2, height - 2, 1.5,
+	                   &fg, NULL, GET_ROUNDED_BUTTONS (style) ? CR_CORNER_ALL : CR_CORNER_NONE);
+	cairo_restore (cr);
 
-	/* XXX: following stuff need rework ... ;-) */
 	cairo_translate (cr, x + 2.0, y + 2.0);
 
 	cairo_scale (cr, (check_size - 4) / 7., (check_size - 4) / 7.);
 
-	fg.a = 1.0;
+	fg.a = GET_REAL_OPACITY (style, 1.0);
 	ge_cairo_set_color (cr, &fg);
+
 	/* draw the interior */
 	if (shadow_type == GTK_SHADOW_IN) {
-		/* copied from GTK+, but it should be more bold ... */
-		cairo_move_to (cr, 7.0, 0.0);
-		cairo_line_to (cr, 7.5, 1.0);
-		cairo_curve_to (cr, 5.3, 2.0, 4.3, 4.0, 3.5, 7.0);
-		cairo_curve_to (cr, 3.0, 5.7, 1.3, 4.7, 0.0, 4.7);
-		cairo_line_to (cr, 0.2, 3.5);
-		cairo_curve_to (cr, 1.1, 3.5, 2.3, 4.3, 3.0, 5.0);
-		cairo_curve_to (cr, 1.0, 3.9, 2.4, 4.1, 3.2, 4.9);
-		cairo_curve_to (cr, 3.5, 3.1, 5.2, 2.0, 7.0, 0.0);
+		cairo_move_to (cr, 0.0, 4.1);
+		cairo_line_to (cr, 2.8, 6.65);
+		cairo_curve_to (cr, 3.7, 5.2, 5.0, 2.65, 6.9, 1.4);
+		cairo_line_to (cr, 6.4, 0.6);
+		cairo_curve_to (cr, 4.5, 1.9, 3.55, 3.4, 2.6, 4.7);
+		cairo_line_to (cr, 0.8, 3);
+		cairo_close_path (cr);
 
 		cairo_fill (cr);
 	} else if (shadow_type == GTK_SHADOW_ETCHED_IN) {
@@ -1148,6 +1347,7 @@ draw_check (GtkStyle * style,
 		cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
 		cairo_move_to (cr, 0.0, 3.5);
 		cairo_line_to (cr, 7.0, 3.5);
+
 		cairo_stroke (cr);
 	}
 
@@ -1186,26 +1386,32 @@ draw_option (GtkStyle * style,
 
 	xc = x + width / 2.0;
 	yc = y + height / 2.0;
-	cairo_arc (cr, xc, yc, radius - 0.5, 0, 2 * M_PI);
+	cairo_arc (cr, xc, yc, radius - 0.5, 0, 2 * G_PI);
 
 	ge_cairo_set_color (cr, &bg);
 	cairo_fill (cr);
 
-	fg.a = 0.5;		/* XXX: #define this? */
+	fg.a = GET_REAL_OPACITY (style, 0.5);	/* XXX: #define this? */
 	ge_cairo_set_color (cr, &fg);
 
 	/* XXX: There should be another shadow ... */
-	cairo_arc (cr, xc, yc, radius - 0.5, -M_PI, M_PI);
-	/* cairo_arc (cr, xc + 0.5, yc + 0.3, radius - 0.3, M_PI - M_PI_4 / 2.0, 3 * M_PI_2 + M_PI_4 / 2.0); */
+	cairo_save (cr);
+	
+	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
+	
+	cairo_arc (cr, xc, yc, radius, 0, 2 * G_PI);
+	cairo_arc (cr, xc + 0.2, yc + 0.2, radius - 1.2, 0, 2 * G_PI);
 
-	cairo_stroke (cr);
+	cairo_fill (cr);
+	
+	cairo_restore (cr);
 
 	if (shadow_type == GTK_SHADOW_IN) {
 		cairo_pattern_t *pattern;
-		fg.a = 1.0;
+		fg.a = GET_REAL_OPACITY (style, 1.0);
 
 		ge_cairo_set_color (cr, &fg);
-		cairo_arc (cr, xc, yc, radius - 3, 0, 2 * M_PI);
+		cairo_arc (cr, xc, yc, radius - 3, 0, 2 * G_PI);
 		cairo_fill (cr);
 
 		pattern =
@@ -1213,19 +1419,27 @@ draw_option (GtkStyle * style,
 
 		bg.a = 0;
 		ge_cairo_pattern_add_color_stop_color (pattern, 0, &bg);
-		bg.a = 0.7;	/* XXX: #define this? */
+		bg.a = GET_REAL_OPACITY (style, 0.7);	/* XXX: #define this? */
 		ge_cairo_pattern_add_color_stop_color (pattern, 1, &bg);
 
 		cairo_set_source (cr, pattern);
 		cairo_pattern_destroy (pattern);
 
 		cairo_move_to (cr, xc, yc);
-		cairo_arc (cr, xc, yc, radius - 4, M_PI - M_PI_4,
-			   3 * M_PI_2 + M_PI_4);
+		cairo_arc (cr, xc, yc, radius - 4, G_PI - G_PI_4,
+			   3 * G_PI_2 + G_PI_4);
 		cairo_close_path (cr);
 		cairo_fill (cr);
+	} else if (shadow_type == GTK_SHADOW_ETCHED_IN) {
+		/* XXX: inconsistent state needs sanity check */
+		cairo_set_line_width (cr, 2.0);
+		cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
+		cairo_move_to (cr, xc - radius + 2, yc);
+		cairo_line_to (cr, xc + radius - 2, yc);
+
+		cairo_stroke (cr);
 	}
-	/* XXX: inconsistent state missing */
+	cairo_destroy (cr);
 }
 
 static GdkPixbuf*
